;----------------------------------------------------------------------------
;
;  adc.asm
;
;  - handle the internal A/D converter
;  - supports the PIC16F873,74, 76 and 77 microcontrollers
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2008 all rights reserved
;
;  22 Jan 2008 A. Cooper
;  - initial version
;
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;  ADCSetup
;
;  - initialize the A/D converter
;
;------------------------------------------------------------------------------
ADCSetup	
	banksel	ADCON1	;setup ADCON1
	movlw	B'10001110'	;channel 0 is analog
			;Vref+=Vcc, Vref-=Gnd 
			;result right justified
	movwf	ADCON1
	bcf	PIE1,ADIE	;A/D interrupt disable
	banksel	ADCON0	;setup ADCON0
	movlw	B'10000001'	;Fosc/32
			;channel 0 selected
			;A/D is on
	movwf	ADCON0
	return


;------------------------------------------------------------------------------
;  ADCRead
;
;  - read the A/D converter
;  - channel (0-7) in w on entry
;
ADCRead	
	pagesel	ADCChan
	call	ADCChan	;set ADC channel
	pagesel	ADCDelay
	call	ADCDelay
	call	ADCGetRead
	call	ADCDelay
	call	ADCGetRead
	banksel	ADRESH	;get result
	movf	ADRESH,w
	banksel	resHi
	movwf	resHi
	banksel	ADRESL
	movf	ADRESL,w
	banksel	resLo
	movwf	resLo
	return

ADCGetRead	
	banksel	ADCON0
	bsf	ADCON0,GO 	;do a conversion
ADCGRLp	btfsc	ADCON0,GO
	goto	ADCGRLp
	return

ADCDelay	movlw	0x10	;do a short delay
	banksel	temp3
	movwf	temp3
ADCDelLp	decfsz	temp3,f
	goto	ADCDelLp
	return

	
;------------------------------------------------------------------------------
;  ADCReadAvg4
;
;  - read the A/D converter
;  - average 4 readings and return
;  - channel (0-7) in w on entry
;
ADCReadAvg4	
	pagesel	ADCChan
	call	ADCChan	;set ADC channel
	call	ADCDelay
	call	ADCGetRead
	call	ADCDelay
	banksel	resHi
	clrf	resHi	;clear results
	clrf	resLo
	movlw	0x04	;set a counter for number of readings
	movwf	count
ADCRdA4Lp	call	ADCGetRead
	banksel	ADRESL	;get result
	movf	ADRESL,w
	banksel	resLo
	addwf	resLo,f
	btfsc	STATUS,C
	 incf	resHi,f
	banksel	ADRESH
	movf	ADRESH,w
	banksel	resHi
	addwf	resHi,f
	decfsz	count,f
	 goto	ADCRdA4Lp
	banksel	resHi
	rrf	resHi,f	;divide by 4 for average
	rrf	resLo,f
	rrf	resHi,f
	rrf	resLo,f
	movlw	0x03
	andwf	resHi,f
	return


;----------------------------------------------------------------------------
;  ADCChan
;
;  - set the channel of the A/D converter
;  - channel number in W on entry
;
ADCChan	
	banksel	temp1
	movwf	temp1	;temp save for shift
	rlf	temp1,f	;shift to position
	rlf	temp1,f
	rlf	temp1,w
	andlw	0x38	;insure parameter in correct bits
	iorlw	B'10000001'	;same A/D setup as above
	banksel	ADCON0	
	movwf	ADCON0	;write result to ADCON0
	return
ADCChan0	movlw	0x00	;set channel to 0
	goto	ADCChan	
ADCChan1	movlw	0x01	;set channel to 1
	goto	ADCChan	
ADCChan2	movlw	0x02	;set channel to 2
	goto	ADCChan	
ADCChan3	movlw	0x03	;set channel to 3
	goto	ADCChan	
ADCChan4	movlw	0x04	;set channel to 4
	goto	ADCChan	
ADCChan5	movlw	0x05	;set channel to 5
	goto	ADCChan	
ADCChan6	movlw	0x06	;set channel to 6
	goto	ADCChan	
ADCChan7	movlw	0x07	;set channel to 7
	goto	ADCChan	


; end of adc.asm
;----------------------------------------------------------------------------
