;----------------------------------------------------------------------------
;
;  host.asm
;
;  - Output routines for com with a serial host
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2000-2013 all rights reserved
;
;  26Apr2010 A.Cooper
;  - Modified for LEDCtrl
;  29Jan2013 A.Cooper
;  - Modified for iSource
;  - Modified for use with PIC16F873 (must banksel all RAM access)
;  - Converted HostChar and HostDChar to macros 
;
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;  HostChar & HostDChar macros
;
;  - send an ASCII char host
;  - char in W
;
HostChar	macro		;send character
	pagesel	ComSend
	call	ComSend
	endm

HostDChar	macro		;send decimal char
	pagesel	ComDSend
	call	ComDSend
	endm


;----------------------------------------------------------------------------
;  HostHex
;
;  - send a hexidecimal byte to host
;  - data in W
;
HostHex	
	banksel	temp4
	movwf	temp4	;temp save char
	swapf	temp4,w	;get upper nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C
	 addlw	0x07
	addlw	0x3A		
	pagesel	ComSend	;send it
	call	ComSend
	banksel	temp4
	movf	temp4,w	;get lower nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C
	 addlw	0x07
	addlw	0x3A		
	pagesel	ComSend	;send it
	call	ComSend
	return


;----------------------------------------------------------------------------
;  HostDec
;
;  - send a byte to host in decimal
;  - data in W
;
HostDec	
	banksel	temp1
	movwf	temp1	;temp save value
	clrf	temp2
HDLp100	incf	temp2,f
	movlw	0x64
	subwf	temp1,f
	btfsc	STATUS,C
	 goto 	HDLp100
	movlw	0x64
	addwf	temp1,f
	decf	temp2,w
	addlw	0x30
	pagesel	ComSend	;send 100's digit
	call	ComSend
	pagesel	HDLp10
	banksel	temp2
	clrf	temp2
HDLp10	incf	temp2,f
	movlw	0x0A
	subwf	temp1,f
	btfsc	STATUS,C
	 goto 	HDLp10
	decf	temp2,w
	addlw	0x30
	pagesel	ComSend	;send 10's digit
	call	ComSend
	banksel	temp1
	movlw	0x0A
	addwf	temp1,w
	addlw	0x30
	pagesel	ComSend	;send 10's digit
	call	ComSend
	return


;----------------------------------------------------------------------------
;  HostSDec
;
;  - send a byte to host in decimal
;  - input in signed byte, send negative sign
;  - data in W
;
HostSDec	
	banksel	temp1
	movwf	temp1	;temp save value
	movlw	0x20	;space?
	btfsc	temp1,7
	 movlw	0x2D	;or negative sign
	pagesel	ComSend
	call	ComSend
	pagesel	HostDec
	banksel	temp
	movf	temp,w
	btfss	temp1,7
	 goto	HostDec
	movf	temp1,w	;inverse number for display
	clrf	temp1
	subwf	temp1,w
	goto	HostDec


;----------------------------------------------------------------------------
;  HostDec2
;
;  - send a byte to host in decimal
;  - send only the lower two digits
;  - data in W
;
HostDec2	
	pagesel	HexToBCD
	call	HexToBCD
	banksel	resLo
	swapf	resLo,w	;send second digit
	andlw	0x0F
	addlw	0x30
	HostChar
	banksel	resLo
	movf	resLo,w	;send third digit
	andlw	0x0F
	addlw	0x30
	HostChar
	return


;----------------------------------------------------------------------------
;  HostLDec
;
;  - send two bytes to host in decimal
;  - data in resHi and resLo on entry
;  - leading zeros suppressed, print space instead
;  - uses all four temp registers
;  - Rountine adapted from PIClist by Rich Leggitt with tweaks by Scott Dattalo
;  and bugfix by Dmitry Kiryashov and Nikolai Golovchenko
;  - see: http://www.piclist.com/techref//microchip/math/radix/b2a-16b5a-rl.htm
;
;
HostLDec	
	banksel	count
	clrf	count	;clear registers
	clrf	temp3
	goto	HLD1kStart
HLDsub10k	incf	count,f
HLD1kStart	movlw	0x10	;=10000 & 255
	subwf	resLo,f
	movlw	0x27	;=10000 >> 8
	skpc
	 movlw	0x28	;=(10000>>8)+1
	subwf	resHi,f
	bc	HLDsub10k
	movf	count,w	;output char
	pagesel	HLDSend
	call	HLDSend
	banksel	count
	movlw	0x0A	;=10
	movwf	count
HLDadd1k	decf	count,f
	movlw	0xE8	;=1000 & 255
	addwf	resLo,f
	movlw	0x03	;=1000 >> 8
	skpnc
	 movlw	0x04	;=(1000>>8)+1
	addwf	resHi,f
	bnc	HLDadd1k
        	movf	count,w	;send char
	pagesel	HLDSend
	call	HLDSend
	banksel	count
	clrf	count
	movlw	0x64	;=100
	goto	HLD100Start
HLDsub100	incf	count,f
HLD100Start	subwf	resLo,f
	skpnc
	 goto	HLDsub100
	decf	resHi,f
	btfss	resHi,7	;Check msb instead of carry for underflow.
	 goto	HLDsub100
			;at this point, resHi = 0xff, and  0 <= resLo <= 99
	movf	count,w	;output char
	pagesel	HLDSend
	call	HLDSend
	banksel	count
	movlw	0x0A	;=10
	movwf	count
HLDadd10	decf	count,f
	addwf	resLo,f
	bnc	HLDadd10
	movf	count,w	;send last chars
	pagesel	HLDSend
	call	HLDSend
	movf	resLo,w
	HostChar
	return

HLDSend	addlw	0x00	;is it zero?
	banksel	temp3
	btfsc	STATUS,Z	;do leading space suppression
	 goto 	HLDSuppress
	bsf	temp3,0	;set stop supressing flag upon first non-zero
	HostDChar
	return
HLDSuppress	btfss	temp3,0	;not supressing? send zero
	 movlw	0xF0	;else send space
	HostDChar
	return


;----------------------------------------------------------------------------
;  HostMessage
;
;  - send text message to the host
;  - message number in W
;
HostMsg	andlw       0x0F        ;qualify input, message #0-15
	banksel	temp3
            movwf	temp3	;store message number
	movlw	high HMText	;reset double byte table pointer
	movwf	temp2
	movlw	low HMText
	movwf	temp1
HM1	incf	temp1,f	;increment table pointer
	btfsc	STATUS,Z
	incf	temp2,f
	movf	temp2,w	;search for correct string
	movwf	PCLATH
	movf	temp1,w
	call	HMText	;get next byte
	sublw	0x00	;is it end of string (null)?
	btfss	STATUS,Z
	goto	HM1
	decfsz	temp3,f	;count strings
	goto	HM1
	
HM2	
	banksel	temp1
	incf	temp1,f	;increment table pointer
	btfsc	STATUS,Z
	incf	temp2,f
	movf	temp2,w	;set upper byte in PCLATH
	movwf	PCLATH
	movf	temp1,w	;get lower byte
	call	HMText	;retrieve value from table
	addlw	0x00	;is it the end of the string?
	btfsc	STATUS,Z
	return
	HostChar		;send it out the serial port
	pagesel	HM2
	goto	HM2

HMText	movwf	PCL
	dt	0x00
HMRevision	equ	0x01
	dt	"Programmable Current Source ver 1.0",0x0D,0x0A
	dt	"Copyright 2002-2013 Andrew Cooper",0x0D,0x0A
	dt	"http://www.darkerview.com",0x0D,0x0A,0x00
HMReset	equ	0x02
	dt	"iSource ready, reset",0x0D,0x0A,0x00
HMPrompt	equ	0x03
	dt	"iSource>",0x00
HMOn	equ	0x04
	dt	"iSource on",0x0D,0x0A,0x00
HMOff	equ	0x05
	dt	"iSource off",0x0D,0x0A,0x00
HMPwr	equ	0x06
	dt	"iSource set to ",0x00
HMPct	equ	0x07
	dt	"%",0x0D,0x0A,0x00
HMTme	equ	0x08
	dt	" to timeout",0x0D,0x0A,0x00
HMTimeout	equ	0x09
	dt	"Timeout! Resetting.",0x0D,0x0A,0x00
HMCurrent	equ	0x0A
	dt	"Current at ",0x00
HMmA	equ	0x0B
	dt	"mA",0x0D,0x0A,0x00
HMLine	equ	0x0C
	dt	0x0D,0x0A,0x00
HMCmdErr	equ	0x0D
	dt	"Command Error!!",0x0D,0x0A,0x00
HMPmErr	equ	0x0E
	dt	"Illegal Parameter, out of range!!",0x0D,0x0A,0x00
HMHelp	equ	0x0F
	dt	"Current iSource Help...",0x0D,0x0A,0x0A
	dt	"  RST     Reset iSource",0x0D,0x0A
	dt	"  SET     Report state",0x0D,0x0A
	dt	"  ONB xx  Set current source on to xx%",0x0D,0x0A
	dt	"  OFF     Set current source off",0x0D,0x0A
	dt	"  CHK     Check iSource",0x0D,0x0A
	dt	"  HLP     This help page",0x0D,0x0A,0x0A,0x00

; end of host.asm
;----------------------------------------------------------------------------
