;----------------------------------------------------------------------------
;
;  misc.asm
;
;  - utility routines
;
;  Andrew Cooper Silicon Owl
;  http://www.darkerview.com
;  Copyright 2000-2003 all rights reserved
;
;  10 Sep 2000 A. Cooper
;  - initial version, includes timer routines and delays
;  23 Nov 2003 A. Cooper
;  - added HextoBCD from TAG-1 code
;
;
;----------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
;  MakeTime
;
;  - Update software timers based on MasterTime
;
MakeTime	movf	mTime,w	;check for 1 second interval
	sublw	clockCal
	btfsc	STATUS,C
	 return
	clrf	mTime
	banksel	sec
	incf	sec,f	;increment 1 second stuff
	movf	sec,w	;check for 1 minute
	sublw	0x3B
	btfsc	STATUS,C
	 return
	clrf	sec
            btfsc       mode,srcStat ;run timeout only if source is off
             return
	decf	toMin,f	;check hour rollover
            incfsz      toMin,w
             return	
            movlw       0x3B
            movwf       toMin
	decf	toHrs,f
            incfsz      toHrs,w     ;stop at zero
             return
            clrf        toHrs
            clrf        toMin
	return


;------------------------------------------------------------------------------
;
;  Delays
;
;  - delay for the specified time interval
;  - these routines do not use the timers, simple loop delays
;  - loops calculated for 20Mz system clock
;  - temp1, temp2, temp3 contents zeroed
;
Del1ms	
	banksel	temp1
	movlw	0x80	;delay for 1000us = 1ms
	movwf	temp1
	movlw	0x07
	movwf	temp2
DL1	decfsz	temp1,f
	 goto	DL1
	decfsz	temp2,f
	 goto	DL1
	return

DelXms	movwf	temp3	;delay for specified time in milliseconds (X>0)
DL3	call	Del1ms
	decfsz	temp3,f
	 goto	DL3
	return

Del10ms	movlw	0x0A	;delay for 10ms
	goto	DelXms

Del100ms	movlw	0x64	;delay for 100ms
	goto	DelXms


;------------------------------------------------------------------------------
;  HexToBCD
;
;  - convert a single hexadecimal byte to binary coded decimal
;  - enter with hex in W register
;  - exit with result in resultHi and resultLo
;
HexToBCD	
	banksel	temp3
	movwf	temp3	;convert number to BCD
	clrf	resHi	;clear results area
	clrf	resLo	
	movlw	0x08	;set loop counter
	movwf	temp4
	
H2BLoop	movf	resLo,w	;check 2SD for >=5
	sublw	0x4F
	btfsc	STATUS,C
	 goto	H2BD3
	movlw	0x30	;add 3 to upper nibble
	addwf	resLo,f

H2BD3	movlw	0x0F	;check LSD for >=5
	andwf	resLo,w
	sublw	0x04
	btfsc	STATUS,C
	 goto	H2BDL
	movlw	0x03
	addwf	resLo,f

H2BDL	rlf	temp3,f	;shift left
	rlf	resLo,f
	rlf	resHi,f
	
	decfsz	temp4,f	;loop 8 times
	 goto	H2BLoop
	movf	resLo,w
	return


;------------------------------------------------------------------------------
;  Mult10 & Div10
;
;  - single byte operands
;  - multiply or divide by 10
;  - integer math, result truncated
;
Div10       
	banksel	temp1
	movwf       temp1       ;divide by 10
            movlw       0x0A
            clrf        count
Div10LP     incf        count,f
            subwf       temp1,f
            btfsc       STATUS,C
             goto       Div10LP
            decf        count,w
            return

Mult10      
	banksel	count
	movwf       count       ;multiply by 10
            movlw       0x0A
            clrf        temp1
Mult10LP    addwf       temp1,f
            decfsz      count,f
             goto       Mult10LP
            movf        temp1,w
            return


;------------------------------------------------------------------------------
;  MultD10
;
;  - multiply by 10
;  - operand limited to range 0-100
;  - single byte operand, double byte result
;  - use 10x = x(8+2) = 8x + 2x
;  - operand in W on entry, result in resHi/resLo
;
MultD10	
	banksel	resLo       ;save operand
	movwf	resLo
	movwf	resHi
	bcf	STATUS,C
	rlf	resHi,w	;mult operand x2
	clrf	resHi	;clear upper byte
	rlf	resLo,f	;mult x8
	rlf	resLo,f
	rlf	resHi,f
	rlf	resLo,f
	rlf	resHi,f
	addwf	resLo,f	;add x2 to result
	btfsc	STATUS,C
	 incf	resHi,f
	return

; end of misc.asm
;----------------------------------------------------------------------------
