;----------------------------------------------------------------------------
;
;  parse.asm
;
;  - parse the serial buffer and extract the command information
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2003-2013 all rights reserved
;
;  26Nov2003 A. Cooper
;  - initial version
;  15Aug2007 A. Cooper
;  - revised for SMS
;  29Jan2013 A. Cooper
;  - revised for iSource
;
;
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;  Parse
;
;  - parse the serial buffer and extract the command information
;  - if the command is illegal an error message is returned
;
Parse	
	banksel	command
	clrf	bufPtr	;clear pointers
	clrf	command
	incf	command,f
	movlw	HIGH PrsTable
	movwf	tblPtrH
	movlw	LOW PrsTable
	movwf	tblPtrL
	incf	tblPtrL,f
	btfsc	STATUS,Z
	 incf	tblPtrH,f

PrsNext	bcf	STATUS,IRP	;get buffer char	
	movf	bufPtr,w
	addlw	comBuf
	movwf	FSR
	bcf	STATUS,IRP
	movf	INDF,w
	movwf	char

	btfss	char,5	;uppercase command if needed
	 goto	PrsNC
	btfss	char,6
	 goto	PrsNC
	bcf	char,5

PrsNC	movf	tblPtrH,w
	movwf	PCLATH
	movf	tblPtrL,w
	pagesel	PrsTable
	call	PrsTable
	subwf	char,w	;check against table
	btfss	STATUS,Z
	 goto	PrsNot

	incf	tblPtrL,f	;increment pointers
	btfsc	STATUS,Z
	 incf	tblPtrH,f
	incf	bufPtr,f
	
	movf	tblPtrH,w
	movwf	PCLATH
	movf	tblPtrL,w	
	call	PrsTable	;check for end of table entry
	addlw	0x00
	btfss	STATUS,Z
	 goto	PrsNext

	bsf	mode,comCmd
	bcf	mode,comPend
	return		;this was the command!

PrsNot	incf	tblPtrL,f	;skip to end of table entry
	btfsc	STATUS,Z
	 incf	tblPtrH,f
	movf	tblPtrH,w
	movwf	PCLATH
	movf	tblPtrL,w
	pagesel	PrsTable	
	call	PrsTable
	addlw	0x00
	btfss	STATUS,Z	
	 goto	PrsNot
	
	incf	tblPtrL,f	;set on start of command
	btfsc	STATUS,Z
	 incf	tblPtrH,f
	movf	tblPtrH,w
	movwf	PCLATH
	movf	tblPtrL,w	
	call	PrsTable
	sublw	0x0A	;check for end of table
	btfsc	STATUS,Z	
	 goto	PrsCmdErr
	
	clrf	bufPtr	;try next command
	incf	command,f
	goto	PrsNext

PrsCmdErr	movf	comBuf,w	;check for empty command
	sublw	0x0D
	btfsc	STATUS,Z
	 goto	PrsNoCmd
	movlw	HMCmdErr	;no valid command, reset for next command
	pagesel	HostMsg
	call	HostMsg
PrsNoCmd	
	banksel	command
	clrf	command
	clrf	comPtr
	bcf	mode,comCmd
	bcf	mode,comPend
	movlw	HMPrompt
	pagesel	HostMsg
	call	HostMsg
	ComLEDOff
	return

;command table
PrsTable	movwf	PCL
	dt	"RST",0x00	;01
	dt	"SET",0x00	;02
	dt	"ONB",0x00	;03
	dt	"OFF",0x00	;04
	dt	"CHK",0x00	;05
	dt	"HLP",0x00	;06
	dt	"XXX",0x00	;07
	dt	"XXX",0x00	;08
	dt	"XXX",0x00	;09
	dt	"XXX",0x00	;0A
	dt	"XXX",0x00	;0B
	dt	"XXX",0x00	;0C
	dt	"XXX",0x00	;0D
	dt	"XXX",0x00	;0E
	dt	"XXX",0x00	;0F
	dt	0x0A


;------------------------------------------------------------------------------
;  ParseDec
;  - get and decode the parameter
;  - the parameter is up to five digits of decimal characters
;  - put the value into two bytes of binary data
;  - if illegal data sent decode what it can else return 0
;
;  10 Oct 2003  A. Cooper
;  - Initial version
;
;  Input:  none
;  Output: none
;  Global: paramH, paramL, result of conversion
;
;------------------------------------------------------------------------------
ParseDec	
	banksel	paramH	;clear results area
	clrf	paramH
	clrf	paramL
	movf	comBuf+4,w	;check 1st char for end of command
	sublw	0x13
	btfsc	STATUS,Z
	 return
	movf	comBuf+4,w	;valid charater 0-9?
	pagesel	PrsValid
	call	PrsValid
	btfss	STATUS,Z
	 return
	movlw	0x30	;subtract ASCII '0'
	subwf	comBuf+4,w
	movwf	paramL

	movf	comBuf+5,w	;check 2nd char for end of command
	sublw	0x13
	btfsc	STATUS,Z
	 return
	movf	comBuf+5,w	;valid charater 0-9?
	pagesel	PrsValid
	call	PrsValid
	btfss	STATUS,Z
	 return
	call	Prsx10	;multiply result by 10
	movlw	0x30	;subtract ASCII '0'
	subwf	comBuf+5,w
	addwf	paramL,f	;add to result
	btfsc	STATUS,C
	 incf	paramH,f

	movf	comBuf+6,w	;check 3rd char for end of command
	sublw	0x13
	btfsc	STATUS,Z
	 return
	movf	comBuf+6,w	;valid charater 0-9?
	call	PrsValid
	btfss	STATUS,Z
	 return
	call	Prsx10	;multiply result by 10
	movlw	0x30	;subtract ASCII '0'
	subwf	comBuf+6,w
	addwf	paramL,f	;add to result
	btfsc	STATUS,C
	 incf	paramH,f

	movf	comBuf+7,w	;check 4th char for end of command
	sublw	0x13
	btfsc	STATUS,Z
	 return
	movf	comBuf+7,w	;valid charater 0-9?
	pagesel	PrsValid
	call	PrsValid
	btfss	STATUS,Z
	 return
	call	Prsx10	;multiply result by 10
	movlw	0x30	;subtract ASCII '0'
	subwf	comBuf+7,w
	addwf	paramL,f	;add to result
	btfsc	STATUS,C
	 incf	paramH,f

	movf	comBuf+8,w	;check 5th char for end of command
	sublw	0x13
	btfsc	STATUS,Z
	 return
	movf	comBuf+8,w	;valid charater 0-9?
	call	PrsValid
	btfss	STATUS,Z
	 return
	pagesel	Prsx10
	call	Prsx10	;multiply result by 10
	movlw	0x30	;subtract ASCII '0'
	subwf	comBuf+8,w
	addwf	paramL,f	;add to result
	btfsc	STATUS,C
	 incf	paramH,f
	return
	
PrsValid	movwf	temp	;is this an ASCII 0-9? zero set if so.
	andlw	0xF0	;check upper nibble
	sublw	0x30
	btfss	STATUS,Z
	 return
	btfss	temp,3	;check lower nibble
	 return
	bcf	STATUS,Z
	btfsc	temp,2
	 return
	btfsc	temp,1
	 return
	bsf	STATUS,Z
	return

Prsx10	movlw	0x09	;multiply result registers by 10
	movwf	count
	movf	paramH,w	;store value
	movwf	tempH
	movf	paramL,w
	movwf	tempL
Prsx10a	movf	tempL,w	;do it 10 times
	addwf	paramL,f
	movf	tempH,w
	btfsc	STATUS,C
	 incf	paramH,f
	addwf	paramH,f
	decfsz	count,f
	 goto	Prsx10a
	return

; end of parse.asm
;----------------------------------------------------------------------------
