;----------------------------------------------------------------------------
;
;  sourceCmd.asm
;
;  - execute the requested Source command
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2007-2013 all rights reserved
;
;  15 Aug 2007 A. Cooper
;  - initial version
;  29 Jan 2012 A. Cooper
;  - modified for iSource
;
;
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;  SourceCmd
;
;  - excute a parsed command
;
SourceCmd	
	banksel	command
	movlw	HIGH SourceJmp ;do a computed goto
	movwf	PCLATH
	movf	command,w
	addlw	LOW SourceJmp
	btfsc	STATUS,C
	incf	PCLATH,f
SourceJmp	movwf	PCL
	goto	SourceReset	;01 Full Reset
	goto	SourceShow	;02 Show Setup
	goto	SourceOn	;03 Turn On Source
	goto	SourceOff	;04 Turn Off Source
	goto	SourceCheck	;05 Check Source
	goto        SourceHelp	;06 Display Help
	goto	SourceSpare	;07
	goto	SourceSpare	;08 
	goto	SourceSpare	;09
	goto	SourceSpare	;0A
	goto	SourceSpare	;0B
	goto	SourceSpare	;0C
	goto	SourceSpare	;0D
	goto	SourceSpare	;0E 
	goto	SourceSpare	;0F 
SourceSpare	return


;----------------------------------------------------------------------------
;  SourceReset
;
;  - reset the controller
;
SourceReset	goto	Init


;----------------------------------------------------------------------------
;  SourceShow
;
;  - print out all controller configuration settings
;
SourceShow	
            pagesel	HostMsg     ;send Source on message
	banksel	mode
	movlw	HMOn
            btfss       mode,srcStat
             movlw      HMOff        
	call	HostMsg
            movlw       HMPwr       ;send Source power message
            call        HostMsg
            pageSel     HostDec
            movf        power,w
            call        HostDec
            pagesel     HostMsg
            movlw       HMPct
            call        HostMsg
            banksel     toHrs       ;send timeout
            movf        toHrs,w
            pagesel     HostDec2
            call        HostDec2
            movlw       ':'
            HostChar
            banksel     toMin
            movf        toMin,w
            pagesel     HostDec2
            call        HostDec2
            movlw       HMTme
            pagesel     HostMsg
            call        HostMsg
	return


;----------------------------------------------------------------------------
;  SourceHelp
;
;  - print command help
;
SourceHelp	movlw	HMHelp	;send help
	pagesel	HostMsg
	call	HostMsg
	return


;----------------------------------------------------------------------------
;  SourceOn
;
;  - set source to specified power level 1-100%
;
SourceOn	
            pagesel     ParseDec    ;get the parameter
            call        ParseDec
	banksel	paramH
            movf        paramH,w    ;check parameter range
            btfss       STATUS,Z
             goto       SourceOnIP
            movf        paramL,w
            sublw       0x66
            btfss       STATUS,C
             goto       SourceOnIP
            movf        paramL,w
            sublw       0x09
            btfsc       STATUS,C
             goto       SourceOnIP
            bsf         mode,srcStat ;set mode flag
            movf        paramL,w    ;save parameter
            movwf       power
            banksel     ctrlDir	;set control bits
            bcf         ctrlDir,srcEnb
	banksel	ctrlPort
	bsf	ctrlPort,srcEnb
	banksel	power
	movf	power,w	;set source DAC
	pagesel	MultD10	;power x10
	call	MultD10
	movf	resHi,w
	banksel	paramH
	movwf	paramH
	movf	resLo,w
	movwf	paramL
	pagesel	DASet	;send to DAC
	movlw	0x00
	call	DASet
            pagesel	HostMsg     ;send source on message
	movlw	HMOn        
	call	HostMsg
            movlw       HMPwr       ;send source power message
            call        HostMsg
            pageSel     HostDec
	banksel	power
            movf        power,w
            call        HostDec
            pagesel     HostMsg
            movlw       HMPct
            call        HostMsg
            banksel     toMin       ;zero timeout timer
            clrf        toMin
            clrf        toHrs
	return
SourceOnIP	movlw       HMPmErr     ;illegal parameter
            pagesel     HostMsg
            call        HostMsg
            return


;----------------------------------------------------------------------------
;  SourceOff
;
;  - turn off source
;
SourceOff	
	banksel	mode
	bcf         mode,srcStat ;set mode flag
            banksel     ctrlDir	;insure bit is output
            bcf         ctrlDir,srcEnb
            banksel     ctrlPort	;set enable bit off
            bcf         ctrlPort,srcEnb
	banksel	power
            clrf        power       ;show power off
	pagesel	DAZero	;set DAC off
	call	DAZero
	pagesel	HostMsg     ;send message
	movlw	HMOff
	call	HostMsg
            banksel     toHrs	;set timeout
            movlw       0x35
            movwf       sec
            movlw       toDefHrs
            movwf       toHrs
            movlw       toDefMin
            movwf       toMin
	return


;----------------------------------------------------------------------------
;  SourceCheck
;
;  - return source status, on/off and readback current only
;
SourceCheck
	pagesel	HostMsg     ;send Source on message
	movlw	HMOn
            btfss       mode,srcStat
             movlw      HMOff        
	call	HostMsg
            movlw       HMPwr       ;send source power message
            call        HostMsg
            pageSel     HostDec
	banksel	power
            movf        power,w
            call        HostDec
            pagesel     HostMsg
            movlw       HMPct
            call        HostMsg
            pagesel     HostMsg	;send actual current
            movlw       HMCurrent
            call        HostMsg
	movlw	0x00	;read channel 0
	pagesel	ADCReadAvg4
	call	ADCReadAvg4
	pagesel	HostLDec	;send result
            pagesel     HostMsg
            movlw       HMmA
            call        HostMsg
	return

; end of SourceCmd.asm
;----------------------------------------------------------------------------
